from icemac.addressbook.interfaces import IAddressBook
import zope.contentprovider.provider


class AddressBookTitle(zope.contentprovider.provider.ContentProviderBase):
    """Content provider for the addressbook title string."""

    default_title = u'icemac.addressbook'

    def render(self):
        address_book = IAddressBook(self.context)
        if not IAddressBook.providedBy(address_book):
            return self.default_title
        return address_book.title or self.default_title



from icemac.addressbook.i18n import _
import icemac.addressbook.interfaces
import zope.interface


class StartpageData(object):
    """Data of a startpage."""

    zope.interface.implements(icemac.addressbook.interfaces.IStartpageData)

    def __init__(self, iface_name, view, title):
        self.iface_name = iface_name
        self.view = view
        self.title = title

    def __call__(self, *args):
        return self


welcome = StartpageData(
    'icemac.addressbook.interfaces.IAddressBook', 'welcome.html',
    _('Welcome page'))
person_list = StartpageData(
    'icemac.addressbook.interfaces.IAddressBook', 'person-list.html',
    _('Person list'))
search = StartpageData(
    'icemac.addressbook.interfaces.IAddressBook', 'search.html',
    _('Search'))



# Copyright (c) 2009-2014 Michael Howitz
# See also LICENSE.txt
import z3c.menu.ready2go
import zope.interface


class IMasterData(z3c.menu.ready2go.ISiteMenu):
    """Menu containing viewlets which provide links to edit the master data."""


class IMasterDataMenuItemOn(zope.interface.Interface):
    """List of view names for which the master data menu item in the main
       navigation should be highlighted."""



# -*- coding: utf-8 -*-
import collections
import datetime
import gocept.jslint
import icemac.addressbook.addressbook
import icemac.addressbook.interfaces
import icemac.addressbook.person
import icemac.addressbook.utils
import lxml.etree
import os
import plone.testing
import plone.testing.zca
import pytz
import transaction
import z3c.etestbrowser.wsgi
import zope.app.publication.zopepublication
import zope.component
import zope.component.hooks
import zope.testbrowser.browser
import zope.testbrowser.interfaces


ZODBConnection = collections.namedtuple(
    'ZODBConnection', ['connection', 'rootFolder', 'zodb'])


def createZODBConnection(zodbDB):
    """Create an open ZODB connection."""
    connection = zodbDB.open()
    rootFolder = connection.root()[
        zope.app.publication.zopepublication.ZopePublication.root_name]
    return ZODBConnection(connection, rootFolder, zodbDB)


# Layer factories


def SecondaryZCMLLayer(name, module, package, bases, filename="ftesting.zcml"):
    """Factory to create a new ZCML test layer above an existing one."""
    return plone.testing.zca.ZCMLSandbox(
        name="%sZCML" % name, bases=bases, filename=filename,
        module=module, package=package)


# Test cases

class JSLintTest(gocept.jslint.TestCase):
    """Base test class for JS lint tests."""

    jshint_command = os.environ.get('JSHINT_COMMAND', '/bin/false')
    options = (gocept.jslint.TestCase.options + (
               'evil',
               'eqnull',
               'multistr',
               'sub',
               'undef',
               'browser',
               'jquery',
               'devel'
               ))


# List of users those passwords are not equal to the login name:
USERNAME_PASSWORD_MAP = dict(mgr='mgrpw')


class Browser(z3c.etestbrowser.wsgi.ExtendedTestBrowser):
    """Enriched test browser."""

    ROOT_URL = 'http://localhost'
    # The login URL starts with this string:
    LOGIN_BASE_URL = 'http://localhost/ab/@@loginForm.html?camefrom=http'

    ADDRESS_BOOK_DEFAULT_URL = 'http://localhost/ab'
    ADDRESS_BOOK_WELCOME_URL = 'http://localhost/ab/@@welcome.html'
    ADDRESS_BOOK_EDIT_URL = 'http://localhost/ab/@@edit-address_book.html'
    ADDRESS_BOOK_DELETE_PERSONS_URL = (
        'http://localhost/ab/@@delete-address_book-content.html')
    ADDRESS_BOOK_DELETE_URL = 'http://localhost/ab/@@delete-address_book.html'

    MASTER_DATA_URL = 'http://localhost/ab/@@masterdata.html'

    ENTITIES_EDIT_URL = 'http://localhost/ab/++attribute++entities'
    ENTITY_PERSON_LIST_FIELDS_URL = (
        'http://localhost/ab/++attribute++entities/'
        'icemac.addressbook.person.Person')
    ENTITY_PERSON_ADD_FIELD_URL = (
        'http://localhost/ab/++attribute++entities/'
        'icemac.addressbook.person.Person/@@addField.html')
    ENTITIY_PERSON_EDIT_FIELD_URL = (
        'http://localhost/ab/++attribute++entities/'
        'icemac.addressbook.person.Person/Field-1')
    ENTITIY_PERSON_DELETE_FIELD_URL = (
        'http://localhost/ab/++attribute++entities/'
        'icemac.addressbook.person.Person/Field-1/@@delete.html')

    KEYWORDS_LIST_URL = 'http://localhost/ab/++attribute++keywords'
    KEYWORD_ADD_URL = (
        'http://localhost/ab/++attribute++keywords/@@addKeyword.html')
    KEYWORD_EDIT_URL = 'http://localhost/ab/++attribute++keywords/Keyword'
    KEYWORD_DELETE_URL = (
        'http://localhost/ab/++attribute++keywords/Keyword/@@delete.html')

    PERSON_ADD_URL = 'http://localhost/ab/@@addPerson.html'
    PERSON_CLONE_URL = 'http://localhost/ab/Person/@@clone.html'
    PERSON_EDIT_URL = 'http://localhost/ab/Person'
    PERSONS_LIST_URL = 'http://localhost/ab/@@person-list.html'
    PERSON_DELETE_URL = 'http://localhost/ab/Person/@@delete_person.html'
    PERSON_DELETE_ENTRY_URL = (
        'http://localhost/ab/Person/@@delete_entry.html')
    PERSON_EXPORT_URL = 'http://localhost/ab/Person/@@export.html'

    POSTAL_ADDRESS_DELETE_URL = (
        'http://localhost/ab/Person/PostalAddress-2/@@delete.html')
    PHONE_NUMBER_DELETE_URL = (
        'http://localhost/ab/Person/PhoneNumber-2/@@delete.html')
    EMAIL_ADDRESS_DELETE_URL = (
        'http://localhost/ab/Person/EMailAddress-2/@@delete.html')
    HOMEPAGE_ADDRESS_DELETE_URL = (
        'http://localhost/ab/Person/HomePageAddress-2/@@delete.html')

    FILE_ADD_URL = 'http://localhost/ab/Person/@@addFile.html'
    FILE_DELETE_URL = 'http://localhost/ab/Person/File/@@delete.html'
    FILE_DOWNLOAD_URL = 'http://localhost/ab/Person/File/download.html'

    PRINCIPALS_LIST_URL = 'http://localhost/ab/++attribute++principals'
    PRINCIPAL_ADD_URL = (
        'http://localhost/ab/++attribute++principals/@@addPrincipal.html')
    PRINCIPAL_EDIT_URL = 'http://localhost/ab/++attribute++principals/2'
    PRINCIPAL_EDIT_URL_1 = 'http://localhost/ab/++attribute++principals/1'
    PRINCIPAL_DELETE_URL = (
        'http://localhost/ab/++attribute++principals/2/@@delete_user.html')
    PRINCIPAL_DELETE_URL_1 = (
        'http://localhost/ab/++attribute++principals/1/@@delete_user.html')

    SEARCH_URL = 'http://localhost/ab/@@search.html'
    SEARCH_BY_NAME_URL = 'http://localhost/ab/@@name_search.html'
    SEARCH_BY_KEYWORD_URL = 'http://localhost/ab/@@multi_keyword.html'
    SEARCH_MULTI_UPDATE_URL = 'http://localhost/ab/@@multi-update'
    SEARCH_MULTI_UPDATE_CHOOSE_FIELD_URL = (
        'http://localhost/ab/multi-update/chooseField')
    SEARCH_MULTI_UPDATE_ENTER_VALUE_URL = (
        'http://localhost/ab/multi-update/enterValue')
    SEARCH_MULTI_UPDATE_CHECK_RESULT_URL = (
        'http://localhost/ab/multi-update/checkResult')
    SEARCH_MULTI_UPDATE_COMPLETE_URL = (
        'http://localhost/ab/@@multi-update-completed')
    SEARCH_DELETE_URL = 'http://localhost/ab/@@delete_persons.html'

    PREFS_URL = 'http://localhost/ab/++preferences++/ab'
    PREFS_TIMEZONE_URL = (
        'http://localhost/ab/++preferences++/ab.timeZone/@@index.html')

    def login(self, username, password=None):
        """Login a user using basic auth."""
        if password is None:
            password = USERNAME_PASSWORD_MAP.get(username, username)
        self.addHeader(
            'Authorization', 'Basic {username}:{password}'.format(
                username=username, password=password))
        return self

    def formlogin(self, username, password, use_current_url=False):
        """Login using the login form."""
        if not use_current_url:
            self.open(self.ADDRESS_BOOK_DEFAULT_URL)
        self.getControl('User Name').value = username
        self.getControl('Password').value = password
        self.getControl('Log in').click()
        assert 'You have been logged-in successfully.' == self.message
        return self

    def lang(self, lang):
        """Set the language for the browser."""
        self.addHeader('Accept-Language', lang)

    def open(self, url):
        super(Browser, self).open(url)
        return self

    @property
    def contents_without_whitespace(self):
        """Browser contents but with removed whitespace."""
        return self.contents.replace(' ', '').replace('\n', '')

    def etree_to_list(self, etree):
        """"Convert an etree into a list (lines without leading whitespace.)"""
        return [x.strip()
                for x in lxml.etree.tostring(etree).split('\n')
                if x.strip()]

    @property
    def message(self):
        """Return the info messages displayed in browser.

        Returns string when there is exactly one message, list otherwise.
        """
        messages = [x.text
                    for x in self.etree.xpath(
                        '//div[@id="info-messages"]/ul/li')]
        if len(messages) == 1:
            return messages[0]
        return messages

    def keyword_search(self, keyword, apply=None):
        """Search for a keyword via keyword-search.

        If `apply` is not `None` select the value from the dropdown and submit
        the search result handler.

        """
        self.open(self.SEARCH_BY_KEYWORD_URL)
        self.getControl('keywords').displayValue = [keyword]
        self.getControl('Search').click()
        if apply:
            self.getControl('Apply on selected persons').displayValue = [apply]
            self.getControl(name='form.buttons.apply').click()

    @property
    def submit_control_names(self):
        """List of the names of the submit controls in the form."""
        return self._get_control_names(
            zope.testbrowser.interfaces.ISubmitControl, self.getForm())

    @property
    def submit_control_names_all_forms(self):
        """List of the names of the submit controls in all forms."""
        forms = [self.getForm(index=x)
                 for x in range(len(list(self.mech_browser.forms())))]
        names = [
            self._get_control_names(
                zope.testbrowser.interfaces.ISubmitControl, x)
            for x in forms]
        return names

    @property
    def all_control_names(self):
        """List of the names of all controls in the form."""
        return self._get_control_names(
            zope.testbrowser.interfaces.IControl, self.getForm())

    def in_out_widget_select(self, control_name, select_controls):
        """Add a selection to an in-out-widget.

        control_name ... name of the in-out-control, something like
                         'form.widgets.columns'
        select_controls ... list of control instances (item controls of in or
                             out list) which should be selected
        """
        form = self.getForm()
        for control in select_controls:
            form.mech_form.new_control(
                type='hidden',
                name='%s:list' % control_name,
                attrs=dict(value=control.optionValue))

    def _get_control_names(self, interface, form):
        """Get a sorted list of names of controls providing `interface`."""
        names = []
        for ctrl in form.mech_form.controls:
            control = zope.testbrowser.browser.controlFactory(ctrl, form, self)
            if interface.providedBy(control):
                names.append(control.name)
        return sorted(names)


class Webdriver(object):
    """Wrapper around Selenese."""

    def __init__(self, selenium):
        self.selenium = selenium

    def login(self, username):
        transaction.commit()
        sel = self.selenium
        sel.open("http://{username}:{password}@{server}/".format(
                 username=username, server=self.selenium.server,
                 password=USERNAME_PASSWORD_MAP.get(username, username)))
        return sel

    @property
    def message(self):
        return self.selenium.getText('css=#info-messages')


# Helper functions to create objects in the database

def create_addressbook(parent, name, title=None):
    """Create an address book in `parent`."""
    ab = icemac.addressbook.utils.create_obj(
        icemac.addressbook.addressbook.AddressBook)
    if title is not None:
        ab.title = title
    parent[name] = ab
    return ab


def create(address_book, parent, interface, set_as_default=False, *args, **kw):
    """Create an object using an entity.

    interface ... interface of the object which should be created.
    """
    with zope.component.hooks.site(address_book):
        entity = icemac.addressbook.interfaces.IEntity(interface)
        name = icemac.addressbook.utils.create_and_add(
            parent, entity.getClass(), *args)
        obj = parent[name]
        for key, value in kw.items():
            field = entity.getField(key)
            context = field.interface(obj)
            field.set(context, value)
        if set_as_default:
            field = icemac.addressbook.person.get_default_field(
                entity.interface)
            field.set(parent, obj)
        zope.lifecycleevent.modified(obj)
    return obj


def set_modified(obj, *args):
    """Set the modification date of an object."""
    dt = datetime.datetime(*args, tzinfo=pytz.utc)
    zope.dublincore.interfaces.IZopeDublinCore(obj).modified = dt
    return dt


def delete_field(browser, field_name):
    """Delete a user defined field."""
    manager = browser.login('mgr')
    manager.open(
        browser.ENTITY_PERSON_LIST_FIELDS_URL +
        '/{}/@@delete.html'.format(field_name))
    manager.getControl('Yes').click()
    assert manager.message.endswith('" deleted.')



# -*- coding: utf-8 -*-
from icemac.addressbook.principals.interfaces import IPasswordFields
from icemac.addressbook.principals.interfaces import IPrincipal, IRoles
from icemac.addressbook.principals.principals import Principal, created
from zope.interface.verify import verifyObject


def test_principals__Principal__1(zcmlS):
    """It fulfills the `IPrincipal` interface."""
    principal = Principal()
    # We need to call the created event handler here, because the person
    # attribute is a descriptor wrapping the object verifyObject expects.
    created(principal, None)
    assert verifyObject(IPrincipal, principal)


def test_principals__Principal__2(zcmlS):
    """It fulfills the `IPasswordFields` interface."""
    principal = Principal()
    assert verifyObject(IPasswordFields, principal)


def test_principals__Principal__3(zcmlS):
    """It fulfills the `IRoles` interface."""
    principal = Principal()
    assert verifyObject(IRoles, principal)



import pytest


def test_session__FlashedSessionCredentialsPlugin__1(address_book, browser):
    """The login screen displays a flash message.

    As cookie authorization is used an unauthorized user is redirected to
    the log-in page.

    """
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert (
        'To log-in enter your username and password and submit the form.' ==
        browser.message)
    assert (
        browser.LOGIN_BASE_URL +
        '%3A%2F%2Flocalhost%2Fab%2F%40%40index.html' == browser.url)


@pytest.mark.parametrize('username, password',
                         (('wrong_username@example.com', '123456789'),
                          ('username@example.com', 'wrong_123456789')))
def test_session__FlashedSessionCredentialsLoginForm__1(
        address_book, UserFactory, browser, username, password):
    """A wrong user name or password at login leads to an error message."""
    UserFactory(address_book, u'Test', u'User', u'username@example.com',
                u'123456789', ['Visitor'])
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getControl('User Name').value = username
    browser.getControl('Password').value = password
    browser.getControl('Log in').click()
    assert ('Login failed. Username and/or password might be wrong.' ==
            browser.message)
    assert 'Please provide Login Information' in browser.contents
    assert (
        browser.LOGIN_BASE_URL +
        '%3A%2F%2Flocalhost%2Fab%2F%40%40index.html' == browser.url)
    # User name and password are not re-displayed if there was an error:
    assert '' == browser.getControl('User Name').value
    assert '' == browser.getControl('Password').value


@pytest.mark.parametrize('role', ('Visitor', 'Editor', 'Administrator'))
def test_session__FlashedSessionCredentialsLoginForm__2(
        address_book, UserFactory, browser, role):
    """A user is able to log-in using the cookie login form."""
    UserFactory(address_book, u'Test', u'User', u'username@example.com',
                u'123456789', [role])
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getControl('User Name').value = 'username@example.com'
    browser.getControl('Password').value = '123456789'
    browser.getControl('Log in').click()
    assert 'You have been logged-in successfully.' == browser.message
    assert browser.ADDRESS_BOOK_WELCOME_URL == browser.url
    browser.getLink('Master data').click()
    browser.getLink('Users').click()
    # Role name is shown in user list:
    assert ('<td>username@example.com</td><td>{}</td>'.format(role) in
            browser.contents_without_whitespace)


@pytest.mark.parametrize('role', ('Visitor', 'Editor', 'Administrator'))
def test_session__FlashedSessionCredentialsLoginForm__3(
        address_book, UserFactory, browser, role):
    """Selecting the ``Logout``-Link leads to the log-in form."""
    UserFactory(address_book, u'Test', u'User', u'username@example.com',
                u'123456789', [role])
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    browser.getControl('User Name').value = 'username@example.com'
    browser.getControl('Password').value = '123456789'
    browser.getControl('Log in').click()
    # Logout:
    browser.getLink('Logout').click()
    assert (
        ['You have been logged out successfully.',
         'To log-in enter your username and password and submit the form.'] ==
        browser.message)
    assert (
        browser.LOGIN_BASE_URL +
        '%3A%2F%2Flocalhost%2Fab%2F%40%40welcome.html' == browser.url)


def test_session__FlashedSessionCredentialsLoginForm__4(
        address_book, AddressBookFactory, UserFactory, browser):
    """Log-in to another address book not possible.

    It is not possible to log in into another addressbook besides the one
    the user was created in (password does not get accepted).

    """
    UserFactory(address_book, u'Ad', u'Min', u'admin@example.com',
                u'123456789', ['Administrator'])
    AddressBookFactory('ab2')
    browser.open('http://localhost/ab2')
    assert ('http://localhost/ab2/@@loginForm.html?camefrom='
            'http%3A%2F%2Flocalhost%2Fab2%2F%40%40index.html' == browser.url)
    browser.getControl('User Name').value = 'admin@example.com'
    browser.getControl('Password').value = '123456789'
    browser.getControl('Log in').click()
    assert ('Login failed. Username and/or password might be wrong.' ==
            browser.message)
    assert ('http://localhost/ab2/@@loginForm.html?camefrom='
            'http%3A%2F%2Flocalhost%2Fab2%2F%40%40index.html' == browser.url)



# Copyright (c) 2008-2014 Michael Howitz
# See also LICENSE.txt
"""I18n message factory for icemac.addressbook. """


import zope.i18nmessageid


_ = MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')



from mechanize import HTTPError
import pytest


def test__layout_pt__1(address_book, browser):
    """layout.pt renders title of the address book in the title tag and h1."""
    address_book.title = u'My addresses'
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert 'My addresses' == browser.title
    assert ['My addresses'] == browser.etree.xpath('//h1/span/text()')


def test__layout_pt__2(address_book, browser):
    """layout.pt falls back to the default title if no title is set."""
    address_book.title = u''
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert 'icemac.addressbook' == browser.title
    assert ['icemac.addressbook'] == browser.etree.xpath('//h1/span/text()')


def test__layout_pt__3(address_book, browser):
    """layout.pt renders the default title on the root page."""
    browser.login('mgr')  # need to log in to avoid HTTP-401
    browser.open(browser.ROOT_URL)
    assert 'icemac.addressbook' == browser.title
    assert ['icemac.addressbook'] == browser.etree.xpath('//h1/span/text()')


def test__layout_pt__4(address_book, browser):
    """layout.pt renders the default FavIcon on the root page."""
    browser.login('mgr')  # need to log in to avoid HTTP-401
    browser.open(browser.ROOT_URL)
    assert 'href="/++resource++img/favicon-red.ico"' in browser.contents


def test__layout_pt__5(address_book, browser):
    """layout.pt renders the selected FavIcon inside the address book."""
    address_book.favicon = u'/++resource++img/favicon-green.ico'
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert 'href="/++resource++img/favicon-green.ico"' in browser.contents


def test__layout_pt__6(address_book, browser):
    """layout.pt renders the NotFound page nicely."""
    with pytest.raises(HTTPError) as err:
        browser.open('http://localhost/I-do-not-exist')
    assert 'HTTP Error 404: Not Found' == str(err.value)
    # If this test fails there was an exception during rendering the error
    # page:
    assert ('The page you are trying to access is not available' in
            browser.contents)


def test__layout_pt__7(address_book, browser):
    """layout.pt renders the Unauthorized page nicely."""
    with pytest.raises(HTTPError) as err:
        browser.open(browser.ROOT_URL)
    assert 'HTTP Error 401: Unauthorized' == str(err.value)
    # If this test fails there was an exception during rendering the error
    # page:
    assert 'You are not authorized.' in browser.contents



# -*- coding: utf-8 -*-
from icemac.addressbook.principals.sources import role_source
import gocept.httpserverlayer.wsgi
import gocept.selenium
import gocept.selenium.wd_selenese
import icemac.addressbook
import icemac.addressbook.file.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.principals.principals
import icemac.addressbook.startup
import icemac.addressbook.testing
import icemac.addressbook.utils
import mock
import os
import os.path
import plone.testing.zodb
import pytest
import tempfile
import transaction
import zope.app.wsgi.testlayer
import zope.component.hooks
import zope.event
import zope.i18n
import zope.processlifetime
import zope.testbrowser.wsgi


CURRENT_CONNECTION = None


# Fixtures to set-up infrastructure which are usable in tests:

@pytest.yield_fixture(scope='function')
def empty_zodb(zodbS):
    """Get the address book from the right demo storage."""
    for connection in pyTestStackDemoStorage(zodbS, 'EmptyZODB'):
        yield connection


@pytest.yield_fixture(scope='function')
def address_book(addressBookConnectionF):
    """Get the address book as site."""
    for address_book in site(addressBookConnectionF):
        yield address_book


@pytest.yield_fixture(scope='function')
def translated_address_book(translationZcmlF):
    """Get the address book as site for translation tests."""
    for zodb in pyTestEmptyZodbFixture():
        for zodb in pyTestAddressBookFixture(zodb, 'TranslatedAddressBook'):
            for conn in pyTestStackDemoStorage(zodb, 'TranslatedAddressBook2'):
                for address_book in site(conn):
                    yield address_book


@pytest.yield_fixture(scope='function')
def person_data(personDataS):
    """Provide predefined person data, see `personDataS`."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            personDataS.zodb, 'PersonFunction'):
        for addressbook in site(connection):
            yield addressbook


@pytest.fixture(scope='function')
def browser(browserWsgiAppS):
    """Fixture for testing with zope.testbrowser."""
    assert CURRENT_CONNECTION is not None, \
        "The `browser` fixture needs a database fixture like `address_book`."
    return icemac.addressbook.testing.Browser(wsgi_app=browserWsgiAppS)


@pytest.fixture(scope='function')
def browser2(browserWsgiAppS):
    """Fixture for testing with a second zope.testbrowser."""
    return icemac.addressbook.testing.Browser(wsgi_app=browserWsgiAppS)


@pytest.fixture(scope='function')
def browser3(browserWsgiAppS):
    """Fixture for testing with a third zope.testbrowser."""
    return icemac.addressbook.testing.Browser(wsgi_app=browserWsgiAppS)


def interpolate_insted_of_translate(self, msgid, mapping=None, *args, **kw):
    """Use interpolation instead of translation."""
    return zope.i18n.interpolate(msgid, mapping)


@pytest.yield_fixture(scope='function')
def webdriver(webdriverS, httpServerS):
    """Fixture to run tests using Webdriver."""
    assert CURRENT_CONNECTION is not None, \
        "The `webdriver` fixture needs a database fixture like `address_book`."
    timeout = int(os.environ.get('GOCEPT_SELENIUM_TIMEOUT', 10))
    webdriver = icemac.addressbook.testing.Webdriver(
        gocept.selenium.wd_selenese.Selenese(
            webdriverS, httpServerS, timeout))
    # Allow any language setting in the Webdriver browser by falling back to
    # interpolation instead of translation:
    translate = 'zope.i18n.translationdomain.TranslationDomain.translate'
    getPreferredLanguages = (
        'zope.publisher.browser.BrowserLanguages.getPreferredLanguages')
    with mock.patch(translate, new=interpolate_insted_of_translate), \
            mock.patch(getPreferredLanguages, return_value=['en-us', 'en']):
        yield webdriver


@pytest.yield_fixture(scope='function')
def tmpfile():
    """Fixture to create a temporary file with a defined content and suffix.

    Returns a callable to create the file which returns a tuple of file handle
    and file name.

    """
    filename = None
    fhr = None

    def tmpfile(content, suffix):
        fd, filename = tempfile.mkstemp(suffix=suffix)
        # It does not seem to be possible to use os.fdopen with 'rw' -- it
        # leads to "[Errno 9] Bad file descriptor"
        with os.fdopen(fd, 'w') as fhw:
            fhw.write(content)
        fhr = file(filename, 'r')
        return fhr, os.path.basename(filename)
    yield tmpfile
    if fhr is not None:
        fhr.close()
        os.unlink(filename)


# Fixtures to help asserting

@pytest.fixture(scope='function')
def sitemenu(browser):
    """Helper fixture to test the selections in the site menu.

    USABLE IN TESTS.

    """
    class SiteMenu(object):
        """Helper class to test selections in the site menu."""

        def __init__(self, browser, menu_item_index, menu_item_title,
                     menu_item_URL):
            """Parameters:

            browser ... return value of the ``browser`` fixture.
            menu_item_index ... zero-based index of the position of the item in
                                 the menu
            menu_item_title ... title of the menu item
            menu_item_URL ... URL the menu item should point to

            """
            self.browser = browser
            self.menu_item_index = menu_item_index
            self.menu_item_title = menu_item_title
            self.menu_item_URL = menu_item_URL

        def item_selected(self, url):
            self.browser.open(url)
            return bool(
                self.browser.etree.xpath(self._xpath)[0].attrib.get('class'))

        def assert_correct_menu_item_is_tested(self):
            self.browser.open(self.menu_item_URL)
            assert self.menu_item_title == self.browser.etree.xpath(
                '%s/a/span' % self._xpath)[0].text

        @property
        def _xpath(self):
            # xpath is one based!
            return '//ul[@id="main-menu"]/li[%s]' % (self.menu_item_index + 1)

    return SiteMenu


@pytest.fixture(scope='function')
def assert_address_book(address_book):
    """Fixture returning an object providing a custom address book asserts."""
    class Assertions:
        """Assertions helpful to check automatic installation routines."""

        def __init__(self, address_book):
            self.address_book = address_book

        def has_local_utility(self, iface, name=''):
            """Assert that there is a local utility for the given interface."""
            util = zope.component.queryUtility(
                iface, context=self.address_book, name=name)
            assert util is not None

        def has_attribute(self, attribute, iface, name=''):
            """Assert the address book has an attribute provided as utility."""
            assert iface.providedBy(getattr(self.address_book, attribute))
            self.has_local_utility(iface, name)

    return Assertions(address_book)


# Fixtures to create objects:

@pytest.fixture(scope='function')
def AddressBookFactory(addressBookConnectionF):
    """Create an address book in the root folder."""
    def create_addressbook(name, title=None):
        return icemac.addressbook.testing.create_addressbook(
            addressBookConnectionF.rootFolder, name, title)
    return create_addressbook


@pytest.fixture(scope='session')
def FieldFactory():
    """Create a user defined field in the address book.

    type ... see values of .sources.FieldTypeSource

    To create values for a Choice field use: values=<list of values>
    """
    def create(address_book, iface, type, title, **kw):
        with zope.component.hooks.site(address_book):
            field = icemac.addressbook.utils.create_obj(
                icemac.addressbook.entities.Field, type=type, title=title,
                **kw)
            entity = icemac.addressbook.interfaces.IEntity(iface)
            return entity.getRawField(entity.addField(field))
    return create


@pytest.fixture(scope='session')
def KeywordFactory():
    """Create a keyword in the address book."""
    def create_keyword(address_book, title, **kw):
        return icemac.addressbook.testing.create(
            address_book, address_book.keywords,
            icemac.addressbook.interfaces.IKeyword, title=title, **kw)
    return create_keyword


@pytest.fixture(scope='session')
def PersonFactory(KeywordFactory):
    """Create a person in the address book."""
    def create_person(address_book, last_name, keywords=[], **kw):
        keywords_set = set()
        keywords_util = zope.component.getUtility(
            icemac.addressbook.interfaces.IKeywords)
        for candidate in keywords:
            if icemac.addressbook.interfaces.IKeyword.providedBy(candidate):
                keyword = candidate
            else:
                keyword = keywords_util.get_keyword_by_title(candidate)
                if keyword is None:
                    keyword = KeywordFactory(address_book, candidate)
            keywords_set.add(keyword)
        return icemac.addressbook.testing.create(
            address_book, address_book, icemac.addressbook.interfaces.IPerson,
            last_name=last_name, keywords=keywords_set, **kw)
    return create_person


@pytest.fixture(scope='session')
def FullPersonFactory(
        PersonFactory, PostalAddressFactory, EMailAddressFactory,
        PhoneNumberFactory, HomepageAddressFactory, KeywordFactory):
    """Create a fully defined person in the address book usable in browser.

    Use <entity_name>__<attrib> to write to numbers and addresses.
    Possible entity names: 'email', 'phone', 'postal', 'homepage'

    The `keywords` parameter accepts an iterable of either keyword objects or
    keyword titles. Not existing keywords are created.

    """
    def create_full_person(address_book, last_name, **kw):
        by_entity = {}
        for key, value in kw.items():
            entity, _, attrib = key.partition('__')
            if not attrib:
                # default to 'person' for not prefixed keys
                attrib = entity
                entity = 'person'
            by_entity.setdefault(entity, {})[attrib] = value

        person = PersonFactory(
            address_book, last_name=last_name, **by_entity.get('person', {}))
        PostalAddressFactory(
            person, set_as_default=True, **by_entity.get('postal', {}))
        EMailAddressFactory(
            person, set_as_default=True, **by_entity.get('email', {}))
        PhoneNumberFactory(
            person, set_as_default=True, **by_entity.get('phone', {}))
        HomepageAddressFactory(
            person, set_as_default=True, **by_entity.get('homepage', {}))
        return person
    return create_full_person


@pytest.fixture(scope='session')
def PostalAddressFactory():
    """Create a postal address for a person in the address book."""
    def create_postal_address(
            person, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IPostalAddress,
            set_as_default=set_as_default, **kw)
    return create_postal_address


@pytest.fixture(scope='session')
def EMailAddressFactory():
    """Create a e-mail address for a person in the address book."""
    def create_email_address(person, email=None, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IEMailAddress,
            email=email, set_as_default=set_as_default, **kw)
    return create_email_address


@pytest.fixture(scope='session')
def PhoneNumberFactory():
    """Create a phone number for a person in the address book."""
    def create_phone_number(person, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IPhoneNumber,
            set_as_default=set_as_default, **kw)
    return create_phone_number


@pytest.fixture(scope='session')
def HomepageAddressFactory():
    """Create a home page address for a person in the address book."""
    def create_home_page_address(person, set_as_default=False, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.interfaces.IHomePageAddress,
            set_as_default=set_as_default, **kw)
    return create_home_page_address


@pytest.fixture(scope='session')
def FileFactory():
    """Create a file for a person in the address book."""
    def create_file(person, filename, **kw):
        return icemac.addressbook.testing.create(
            person.__parent__, person,
            icemac.addressbook.file.interfaces.IFile, name=filename, **kw)
    return create_file


@pytest.fixture(scope='session')
def UserFactory(FullPersonFactory):
    """Create a user in the address book.

    roles ... list of role titles (!)
    """
    def create_user(
            address_book, first_name, last_name, email, password, roles,
            keywords=[], **kw):
        person = FullPersonFactory(
            address_book, first_name=first_name, last_name=last_name,
            email__email=email, keywords=keywords)
        role_factory = role_source.factory
        role_values = role_factory.getValues()
        selected_roles = []
        for role_title in roles:
            for candidate in role_values:
                if role_factory.getTitle(candidate) == role_title:
                    selected_roles.append(candidate)
                    break
            else:
                raise LookupError(
                    'Role title {!r} unknown.'.format(role_title))
        # Cannot use icemac.addressbook.testing() here because `Principal` is
        # not an entity.
        name = icemac.addressbook.utils.create_and_add(
            address_book.principals,
            icemac.addressbook.principals.principals.Principal,
            person=person, password=password, roles=selected_roles)
        return address_book.principals[name]
    return create_user


# Infrastructure fixtures

@pytest.yield_fixture(scope='session')
def zcmlS():
    """Load ZCML on session scope."""
    layer = icemac.addressbook.testing.SecondaryZCMLLayer(
        'AddressBook', __name__, icemac.addressbook, [])
    layer.setUp()
    yield layer
    layer.tearDown()


@pytest.yield_fixture(scope='session')
def zodbS(zcmlS):
    """Create an empty test ZODB."""
    for zodb in pyTestEmptyZodbFixture():
        yield zodb


@pytest.yield_fixture(scope='session')
def addressBookS(zodbS):
    """Create an address book for the session."""
    for zodb in pyTestAddressBookFixture(zodbS, 'AddressBookS'):
        yield zodb


@pytest.yield_fixture(scope='function')
def addressBookConnectionF(addressBookS):
    """Get the connection to the right demo storage."""
    for connection in pyTestStackDemoStorage(addressBookS, 'AddressBookF'):
        yield connection


@pytest.yield_fixture(scope='function')
def translationZcmlF(zcmlS):
    """Load ZCML including locales packages on session scope."""
    layer = icemac.addressbook.testing.SecondaryZCMLLayer(
        'TranslatedAddressBook', __name__, icemac.addressbook, [zcmlS],
        filename='translationtesting.zcml')
    layer.setUp()
    yield layer
    layer.tearDown()


@pytest.fixture(scope='session')
def wsgiAppS():
    """Create a WSGI application. for the current package."""
    zope_appl = icemac.addressbook.startup.zope_application_factory()
    zope_appl.requestFactory = TestHTTPPublicationRequestFactory()
    return icemac.addressbook.startup.application_factory(
        zope_application=zope_appl)


def getRootFolder():
    """`TransactionMiddleware` expects a callable to get the root folder."""
    return CURRENT_CONNECTION.rootFolder


@pytest.fixture(scope='session')
def browserWsgiAppS(wsgiAppS):
    """WSGI application layered with some middlewares for testbrowser."""
    return zope.testbrowser.wsgi.AuthorizationMiddleware(
        zope.app.wsgi.testlayer.TransactionMiddleware(
            getRootFolder, wsgiAppS))


@pytest.yield_fixture(scope='session')
def httpServerS(wsgiAppS):
    """Create a HTTPServer for the WSGI app."""
    server = gocept.httpserverlayer.wsgi.Layer()
    server.wsgi_app = wsgiAppS
    server.setUp()
    server.url = 'http://{}'.format(server['http_address'])
    yield server['http_address']
    server.tearDown()


@pytest.yield_fixture(scope='session')
def webdriverS(httpServerS):
    """Open a browser for webriver tests."""
    layer = gocept.selenium.WebdriverLayer(name='WebdriverLayer')
    layer['http_address'] = httpServerS
    layer.setUp()
    yield layer['seleniumrc']
    layer.tearDown()


@pytest.yield_fixture(scope='session')
def personDataS(
        addressBookS, FullPersonFactory, PostalAddressFactory, KeywordFactory,
        PhoneNumberFactory, EMailAddressFactory, HomepageAddressFactory):
    """Create base data used in person tests."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            addressBookS, 'PersonDataSession'):
        for address_book in icemac.addressbook.conftest.site(connection):
            _create_person(
                address_book, FullPersonFactory, PostalAddressFactory,
                KeywordFactory, PhoneNumberFactory, EMailAddressFactory,
                HomepageAddressFactory)
            yield connection


# Helper functions and classes


def pyTestEmptyZodbFixture():
    """Create an empty ZODB prepared to create an address book inside.

    Yields the zodb object.

    Prepared to be usable with a ``pytest.yield_fixture()``.

    """
    layer = plone.testing.zodb.EmptyZODB()
    layer.setUp()
    zodb = layer['zodbDB']
    zope.event.notify(zope.processlifetime.DatabaseOpened(zodb))
    transaction.commit()
    yield zodb
    layer.tearDown()


def pyTestAddressBookFixture(zodb, name):
    """Create an address book in the ZODB.

    Expects an empty ZODB as created by pyTestEmptyZodbFixture() and a name
    for the demo storage layer.

    Yields the zodb object.

    Prepared to be usable with a ``pytest.yield_fixture()``.

    """
    for connection in pyTestStackDemoStorage(zodb, name):
        icemac.addressbook.testing.create_addressbook(
            connection.rootFolder, name='ab', title=u'test addressbook')
        transaction.commit()
        yield connection.zodb


def pyTestStackDemoStorage(zodb, name):
    """Put a demo storage on top of ``zodb`` and yield the connection tuple.

    Prepared to be usable with a ``pytest.yield_fixture()``.

    """
    global CURRENT_CONNECTION
    storage = plone.testing.zodb.stackDemoStorage(zodb, name=name)
    connection = icemac.addressbook.testing.createZODBConnection(storage)
    transaction.begin()
    CURRENT_CONNECTION = connection
    yield connection
    storage.close()
    transaction.abort()
    connection.connection.close()
    CURRENT_CONNECTION = None


class NonCachingPublicationCache(object):
    """Replacement to prevent caching of ZODB connections."""

    def get(self, key):
        return None

    def __setitem__(self, key, value):
        """Do not cache!"""
        pass


class TestHTTPPublicationRequestFactory(
        zope.app.publication.httpfactory.HTTPPublicationRequestFactory):
    """RequestFactory which uses the global `CURRENT_CONNECTION`."""

    def __init__(self):
        super(TestHTTPPublicationRequestFactory, self).__init__(None)
        self._publication_cache = NonCachingPublicationCache()

    @property
    def _db(self):
        return CURRENT_CONNECTION.zodb

    @_db.setter
    def _db(self, value):
        """Ignoring `value` because the global one should always be used."""


def site(connection):
    """Get the address book from the connection and set it as site.

    Usable in a `yield_fixture`.
    Resets the site to its previous value at exit.

    """
    addressbook = connection.rootFolder['ab']
    with zope.component.hooks.site(addressbook):
        yield addressbook


def _create_person(
        address_book, FullPersonFactory, PostalAddressFactory, KeywordFactory,
        PhoneNumberFactory, EMailAddressFactory, HomepageAddressFactory, **kw):
    """Create a person, usable in tests."""
    KeywordFactory(address_book, u'friend')

    person = FullPersonFactory(
        address_book, u'Tester', first_name=u'Petra',
        keywords=[u'family', u'church'],
        postal__address_prefix=u'c/o Mama', postal__zip=u'88888',
        postal__street=u'Demoweg 23', postal__city=u'Testhausen',
        postal__country=gocept.country.db.Country('AT'),
        phone__number=u'+4901767654321', email__email=u'petra@example.com',
        homepage__url=b'http://petra.example.com', **kw)
    PostalAddressFactory(
        person, address_prefix=u'RST-Software',
        street=u'Forsterstraße 302a', zip=u'98344', city=u'Erfurt')
    PhoneNumberFactory(person, number=u'+4901761234567')
    EMailAddressFactory(person, email=u'pt@rst.example.edu')
    HomepageAddressFactory(person, url='http://www.rst.example.edu')
    transaction.commit()



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2014 Michael Howitz
# See also LICENSE.txt
# $Id$

import icemac.addressbook.browser.personlist
import icemac.addressbook.browser.search.result.base


class PersonTable(
        icemac.addressbook.browser.personlist.BasePersonList,
        icemac.addressbook.browser.search.result.base.BasePersonTable):
    """Result table displaying columns defined in user preferences."""

    values = (
        icemac.addressbook.browser.search.result.base.BasePersonTable.values)


class ExportForm(
        icemac.addressbook.browser.search.result.base.BaseSearchResultForm):
    "Export form showing a table of found results."

    table_class = PersonTable



def test_preferences__1(address_book, webdriver):
    """Testing JS funtions of preferences frontend."""
    sel = webdriver.login('mgr')
    sel.open("/ab/++preferences++/ab")
    # Preference groups are closed by default, fields are not visible:
    sel.assertNotVisible("css=#form-widgets-columns-row")
    # After opening the group the field is shown:
    sel.click("//fieldset[@class='personLists']/legend")
    sel.waitForVisible("css=#form-widgets-columns-row")
    # Clicking on a form element does not close the group:
    sel.addSelection("id=form-widgets-columns-from",
                     "label=person -- birth date")
    sel.click("name=from2toButton")
    sel.assertVisible("css=#form-widgets-columns-row")
    # Clicking on the legend closes the group:
    sel.click("//fieldset[@class='personLists']/legend")
    sel.waitForNotVisible("css=#form-widgets-columns-row")



from ..form import zope_i18n_pattern_to_jquery_pattern
from datetime import datetime
from icemac.addressbook.interfaces import IKeyword
from mock import patch, Mock
from pytz import utc, timezone
import gocept.testing.mock
import icemac.addressbook.browser.form
import pytest
import zope.publisher.browser


@pytest.fixture(scope='function')
def DatetimeDataConverter(mockTimeZone):
    """Set up a `DatetimeDataConverter` object for tests."""
    widget = Mock()
    widget.request = zope.publisher.browser.TestRequest()
    field = Mock()
    field.missing_value = None
    return icemac.addressbook.browser.form.DatetimeDataConverter(field, widget)


@pytest.yield_fixture(scope='module')
def mockTimeZone():
    """Mock the timezone set in preferences to 'Etc/GMT-4'."""
    time_zone = ('icemac.addressbook.browser.form.'
                 'DatetimeDataConverter.time_zone')
    with patch(time_zone, gocept.testing.mock.Property()) as time_zone:
        time_zone.return_value = timezone('Etc/GMT-4')
        yield


def test_form__DatetimeDataConverter__toWidgetValue__1(DatetimeDataConverter):
    """`toWidgetValue` renders datetime with timezone localized."""
    assert u'13/02/01 21:20' == DatetimeDataConverter.toWidgetValue(
        datetime(2013, 2, 1, 17, 20, tzinfo=utc))


def test_form__DatetimeDataConverter__toWidgetValue__2(DatetimeDataConverter):
    """`toWidgetValue` renders naive datetime unchanged."""
    assert u'13/02/01 17:20' == DatetimeDataConverter.toWidgetValue(
        datetime(2013, 2, 1, 17, 20))


def test_form__DatetimeDataConverter__toWidgetValue__3(DatetimeDataConverter):
    """`toWidgetValue` renders `missing_value` as empty string."""
    assert u'' == DatetimeDataConverter.toWidgetValue(None)


def test_form__DatetimeDataConverter__toFieldValue__1(DatetimeDataConverter):
    """`toFieldValue` adds tzinfo."""
    assert (datetime(2013, 2, 1, 21, 20, tzinfo=timezone('Etc/GMT-4')) ==
            DatetimeDataConverter.toFieldValue(u'13/02/01 21:20'))


def test_form__DatetimeDataConverter__toFieldValue__2(DatetimeDataConverter):
    """`toFieldValue` leaves an empty value alone."""
    assert None is DatetimeDataConverter.toFieldValue(u'')


def test_form__zope_i18n_pattern_to_jquery_pattern__1():
    """Function converts German datetime format correctly."""
    assert ('dd.mm.y HH:mm' ==
            zope_i18n_pattern_to_jquery_pattern('datetime', 'dd.MM.yy HH:mm'))


def test_form__zope_i18n_pattern_to_jquery_pattern__2():
    """Function converts American datetime format correctly."""
    assert ('m/d/y hh:mm TT' ==
            zope_i18n_pattern_to_jquery_pattern('datetime', 'M/d/yy h:mm a'))


def test_form__zope_i18n_pattern_to_jquery_pattern__3():
    """Function converts American time format correctly."""
    assert ('h:mm TT' == zope_i18n_pattern_to_jquery_pattern('time', 'h:mm a'))


@pytest.mark.parametrize('datatype', ['Datetime', 'Time'])
def test_form__Widget__1(
        address_book, FieldFactory, KeywordFactory, datatype, webdriver):
    """`DatetimeWidget` renders a JavaScript calendar."""
    FieldFactory(address_book, IKeyword, datatype, u'my-field')
    kw = KeywordFactory(address_book, u'foobar')
    s = webdriver.login('editor')
    s.open('/ab/++attribute++keywords/%s' % kw.__name__)
    # Activate the datetime field which opens the JavaScript calendar
    s.click('id=form-widgets-Field-1')
    # Click the `now` button:
    s.click("//button[@type='button']")
    # And the `done` button:
    s.click("xpath=(//button[@type='button'])[2]")
    # Save the form:
    s.clickAndWait("id=form-buttons-apply")
    # Successful apply leads back to keyword overview:
    assert s.getLocation().endswith('/ab/++attribute++keywords')


def test_form__DateWidget__1(address_book, webdriver):
    """`DateWidget` renders a JavaScript calendar."""
    s = webdriver.login('editor')
    s.open('/ab/@@addPerson.html')
    # Activate the date field which opens the JavaScript calendar
    s.click('id=IcemacAddressbookPersonPerson-widgets-'
            'IcemacAddressbookPersonPerson-birth_date')
    # Click the first of the first month:
    s.click("link=1")
    # Fill in required fiels:
    s.type('id=IcemacAddressbookPersonPerson-widgets-'
           'IcemacAddressbookPersonPerson-last_name', 'Tester')
    # Save the form:
    s.clickAndWait("id=form-buttons-add")
    # Successful apply leads back to keyword overview
    assert s.getLocation().endswith('/ab/@@person-list.html')



def test_resources__1(address_book, browser):
    """Image resources can be delivered to the browser."""
    browser.open('http://localhost/++resource++img/Symbol-Information.png')
    assert '200 Ok' == browser.headers['status']
